import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { useState, useEffect, useMemo, useRef } from "react";
import { colorInts } from "../colors";
import MMGC_INIT from "mmgc1-cpp";
export default (function (_ref) {
  var classPoints = _ref.classPoints,
      regionClsList = _ref.regionClsList,
      imageSrc = _ref.imageSrc,
      imagePosition = _ref.imagePosition,
      _ref$opacity = _ref.opacity,
      opacity = _ref$opacity === void 0 ? 0.5 : _ref$opacity,
      _ref$zIndex = _ref.zIndex,
      zIndex = _ref$zIndex === void 0 ? 2 : _ref$zIndex,
      _ref$position = _ref.position,
      position = _ref$position === void 0 ? "absolute" : _ref$position;
  if (!window.mmgc) window.mmgc = MMGC_INIT();
  var mmgc = window.mmgc;

  var _useState = useState(null),
      _useState2 = _slicedToArray(_useState, 2),
      canvasRef = _useState2[0],
      setCanvasRef = _useState2[1];

  var lastTimeMMGCRun = useRef(0);
  var superPixelsGenerated = useRef(false);

  var _useState3 = useState(),
      _useState4 = _slicedToArray(_useState3, 2),
      sampleImageData = _useState4[0],
      setSampleImageData = _useState4[1];

  useEffect(function () {
    if (!imageSrc) return;
    var canvas = document.createElement("canvas");
    var ctx = canvas.getContext("2d");
    var image = new Image();
    image.crossOrigin = "anonymous";
    image.src = imageSrc;

    image.onload = function () {
      canvas.width = image.naturalWidth;
      canvas.height = image.naturalHeight;
      ctx.drawImage(image, 0, 0);
      var imageData = ctx.getImageData(0, 0, image.naturalWidth, image.naturalHeight);
      superPixelsGenerated.current = false;
      setSampleImageData(imageData);
    };
  }, [imageSrc]);
  useEffect(function () {
    if (!canvasRef) return;
    if (!sampleImageData) return;
    if (classPoints.filter(function (cp) {
      return cp.cls;
    }).length < 3) return;
    if (!mmgc.setImageSize) return; // NEEDS DEBOUNCE

    if (Date.now() < lastTimeMMGCRun.current + 500) return;
    lastTimeMMGCRun.current = Date.now();
    var context = canvasRef.getContext("2d");

    if (!superPixelsGenerated.current) {
      superPixelsGenerated.current = "processing";
      mmgc.setImageSize(sampleImageData.width, sampleImageData.height);
      var imageAddress = mmgc.getImageAddr();
      mmgc.HEAPU8.set(sampleImageData.data, imageAddress);
      mmgc.computeSuperPixels();

      for (var i = 0; i < colorInts.length; i++) {
        mmgc.setClassColor(i, colorInts[i]);
      }

      superPixelsGenerated.current = "done";
    }

    if (superPixelsGenerated.current !== "done") return; // mmgc.setVerboseMode(true)

    mmgc.clearClassPoints();
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = classPoints[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var classPoint = _step.value;
        if (!classPoint.cls) continue;
        if (classPoint.x < 0 || classPoint.x >= 1) continue;
        if (classPoint.y < 0 || classPoint.y >= 1) continue;
        var clsIndex = regionClsList.indexOf(classPoint.cls);

        if (clsIndex > colorInts.length) {
          console.log("Too many classes to draw on mask!");
          continue;
        }

        mmgc.addClassPoint(clsIndex, Math.floor(classPoint.y * sampleImageData.height), Math.floor(classPoint.x * sampleImageData.width));
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    mmgc.computeMasks();
    var maskAddress = mmgc.getColoredMask();
    var cppImDataUint8 = new Uint8ClampedArray(mmgc.HEAPU8.buffer, maskAddress, sampleImageData.data.length // sampleImageData.width * sampleImageData.height * 4
    );
    var maskImageData = new ImageData(cppImDataUint8, sampleImageData.width, sampleImageData.height);
    context.clearRect(0, 0, sampleImageData.width, sampleImageData.height);
    context.putImageData(maskImageData, 0, 0);
  }, [canvasRef, sampleImageData, JSON.stringify(classPoints.map(function (c) {
    return [c.x, c.y, c.cls];
  }))]);
  var style = useMemo(function () {
    var width = imagePosition.bottomRight.x - imagePosition.topLeft.x;
    var height = imagePosition.bottomRight.y - imagePosition.topLeft.y;
    return {
      imageRendering: "pixelated",
      left: imagePosition.topLeft.x,
      top: imagePosition.topLeft.y,
      width: isNaN(width) ? 0 : width,
      height: isNaN(height) ? 0 : height,
      zIndex: zIndex,
      position: position,
      opacity: opacity,
      pointerEvents: "none"
    };
  }, [imagePosition.topLeft.x, imagePosition.topLeft.y, imagePosition.bottomRight.x, imagePosition.bottomRight.y, zIndex, position, opacity]);
  return React.createElement("canvas", {
    style: style,
    width: sampleImageData ? sampleImageData.width : 0,
    height: sampleImageData ? sampleImageData.height : 0,
    ref: setCanvasRef
  });
});